/*!
* Debug JavaScript Library v0.0.1
*
* Copyright (c) 2009 Seth Juarez, http://www.sethjuarez.com
* 
* Permission is hereby granted, free of charge, to any person obtaining
* a copy of this software and associated documentation files (the
* "Software"), to deal in the Software without restriction, including
* without limitation the rights to use, copy, modify, merge, publish,
* distribute, sublicense, and/or sell copies of the Software, and to
* permit persons to whom the Software is furnished to do so, subject to
* the following conditions:
* 
* The above copyright notice and this permission notice shall be
* included in all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
* LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
* OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
* WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
* Date: Mon, 17 Aug 2009
* Revision: 2
*
* Depends:
*  jQuery
*  jQuery UI (draggable)
*
*/

// for global debug settings checker
window.isDebug = true;

(function(){
	// a-la Jquery
	var window = this;

	var debug = window.debug = window._ = function(o) {
		// yes, jQuery is cool
		return new debug.fn.init(o);
	};


	debug.fn = debug.prototype = {
		
		// init function
		init: function(o) {
			target = o;
			if($('#' + this.console).length == 0) {
				this.init_console();
			}

			return this;
		},

		init_console: function() {
			var console = this.console;
			var toggle = this.toggle;
			$('body')
				.append('<div id="' + this.toggle + '">Hide Console</div>');
			$('body')
				.append('<div id="' + this.console 
							+ '"><div class="head">JavaScript Debug Console v0.2</div><ol id="' 
							+ this.nodes + '"></ol></div>');

			//$('#' + console).draggable();
			$('#' + console).css('height', $(window).height() - 20);
			$(window).resize(function() {
				$('#' + console).css('height', $(window).height() - 20);
			});

			$('#' + console)
				.hover(
					function() {
						$(this).fadeTo('fast', 1);
					},
					function() {
						$(this).fadeTo('fast', 0.50);
					}
				);

			$('#' + this.toggle)
				.fadeTo('fast', 0.35)
				.text('Hide Console')
				.hover(
					function() {
						$(this).fadeTo('fast', 1);
					},
					function() {
						$(this).fadeTo('slow', 0.35);
					}
				)
				.click(function() {
					if($('#' + console).is(':hidden')) {
						$('#' + toggle).text('Hide Console');
						$('#' + console).fadeTo('fast', 0.50);
						$('#' + console).show();
					} else {
						$('#' + toggle).text('Show Console');
						$('#' + console).fadeTo('slow', 0);
						$('#' + console).hide();
					}
				});
		},

		// object under inspection
		target: null,
		// id of canvas to-be-used (can change it...)
		console: 'debug_console',
		// canvas jQuery element
		toggle: 'debug_toggle',
		// ol nodes
		nodes: 'debug_nodes',

		// for discovering ominous identity of 'object'
		typeOf: function(v) {
            if (typeof (v) == 'object') {
                if (v === null) return 'null';
                if (v.constructor == (new Array).constructor) return 'array';
                if (v.constructor == (new Date).constructor) return 'date';
                return 'object';
            }
            return typeof (v);
        },
		// write out array
		writeArray: function(li, arg, tab) {
			if(tab) $(li).append('<span class="bcurly">(</span>');
			for(var i = 0; i < arg.length; i++) {
				if(tab) {
					$(li).append('<br/>');
					$(li).append('&nbsp;&nbsp;&nbsp;&nbsp;<span class="index">[' + i + ']:</span>')
					this.dispatcher(li, arg[i], true);
				} else {
					$(li).append('<span class="index">[' + i + ']:</span>')
					this.dispatcher(li, arg[i], true);
					$(li).append('<br/>');
				}
			}
			if(tab) $(li).append('&nbsp;&nbsp;&nbsp;&nbsp;<br/><span class="ecurly">)</span>');
		},

		// write out object
		writeObject: function(li, arg) {
			$(li).append('<span class="bcurly">{</span>');
			var first = true;
			for(var item in arg) {
				if(!first)
					$(li).append(',&nbsp;');

				$(li).append('<span class="prop">' + item + ':</span>');
				this.dispatcher(li, arg[item], true);
				first = false;
			}
			$(li).append('<span class="ecurly">}</span>');
		},

		// write out plain value
		writeVal: function(li, arg) {
			if(typeof(arg) == 'function') {
				var item = arg.toString().replace('function', '<span class="function">function</span>')
								.replace('{', '<span class="bcurly">{</span>')
								.replace('}', '<span class="ecurly">}</span>');
				$(li).append('<span class="val">' + item + '</span>');
			} else {
				$(li).append('<span class="val">' + arg + '</span>');
			}
		},

		// dispatcher that decides 
		// how to write current arg
		dispatcher: function(li, arg, tabbed) {
			var tabbed = (arguments.length == 3) ? tabbed : false;
			var t = this.typeOf(arg);
			if(t == 'array')
				this.writeArray(li, arg, tabbed);
			else if(t == 'object')
				this.writeObject(li, arg);
			else
				this.writeVal(li, arg);
		},

		// initiate write
		write: function(arg) {
			var li = document.createElement('li');

			if(arguments.length == 1)
				$(li).append('<div class="title">' + arg + '</div>');

			this.dispatcher(li, target);
			
			$('#' + this.nodes).append(li);
			$('#' + this.console).animate({ scrollTop: $('#' + this.console).attr('scrollHeight') } , 100);

			return this;
		},

		set: function(o) {
            target = o;
            return this;
        },

        clear: function() {
            $('#' + this.nodes).html('');
            return this;
        },

        version: function() {
            return '0.0.2';
        }
	};

	debug.fn.init.prototype = debug.fn;

})();